# Denying source IP

This directory contains files used for the Denying source IP
workshop. The goal of this workshop is to optimize ACL based on source
IP and also visualize the differences between HTTP and HTTPS.

## Description

There is a service which requires access from only a specific IP range
(10.0.0.0/24 in our example). To limit access a connection rejection
is made by HAProxy using the `src` fetch sample. If the source IP is
not from an authorized network, the connection is closed.


## Performance measurements

The `http-request` frontend is our initial setup and will be our
reference for all measurements using `h1load`.

This frontend uses `http-request deny` to block unauthorized
clients. Since this directive does not close the connection, a
`Connection: close` header is added after the response. HAProxy
understands the connection has to be closed and closes it right after
the response is sent to the client.

This example is a bit more tricky to measure because the connection is
closed and a new connection has to be open for each request (there is
no connection keep-alive).

Connection rejections are difficult to measure because:

* A new TCP connection is made for each request.
* We need to check connections per second instead of requests.
* Results may be inconsistent when running on one core.
* The client and the server have to be saturated.
* Lock congestion happens on the client in SSL context.

To get the best results the number of concurrent connections is raised
from 64 to 128. Preliminary tests showed that network card
interruption saturated one CPU core. These interrupts can be spread
over 2 CPU cores instead 1. Cores 6 and 7 are used:

```
for i in $(grep eth2 /proc/interrupts | cut -f 1 -d:); do
    echo c0 > /proc/irq/$i/smp_affinity
done
```

Start the proxy with command:

```
haproxy -f haproxy.cfg
```

And run the injector successively to benchmark HTTP, HTTPS using
2048-bit RSA key and HTTPS using ECDSA key:

```
taskset -c 0-3 ./h1load -t 4 -c 128 -d 20 -S 'http://198.18.0.102:8000/'
taskset -c 0-3 ./h1load -t 4 -c 128 -d 20 -S 'https://198.18.0.102:8010/'
taskset -c 0-3 ./h1load -t 4 -c 128 -d 20 -S 'https://198.18.0.102:8020/'
```

Here the references is are:

* HTTP: 74k req/s.
* HTTPS (RSA key): <255 req/s.
* HTTPS: (ECDSA key): 2.5k req/s.

It's important to **NOT COMPARE** those results to each other because
they are completely unrelated. They are just reference to compare with
other blocking methods.

An other point to consider is that SSL context on the client side may
also slow down the connection rate.

## A word about rule processing order

HAProxy handles several rules in a specific order. For a request the
order is:

* `tcp-request connection`: Immediately after acceptance of a new
  incoming connection. (Layer 4)
* `tcp-request session`: Once a session is validated, after all
  handshakes have been completed. (Layer 5)
* `tcp-request content`: Can access to the request payload. Requires
  `tcp-inspect delay`. (Layer 7)
* `http-request`: After parsing the request. (Layer 7)

And for a response the order is:

* `tcp-response content`: Can access to the response payload. Requires
  `tcp-response-inspect delay`. (Layer 7)
* `http-response`: After parsing the response. (Layer 7)
* `http-after-response`: Before sending the response to the
  client. (Layer 7)

## Optimization

Instead of filtering the client IP at `http-request` level, we can
filter it at `tcp-request` level.

We can run benchmark for all `tcp-request` and for HTTP, HTTPS with
RSA key and HTTPS with ECDSA key. For that we just need to change the
connection port:

* `8001`: `tcp request content` in HTTP (123k req/s).
* `8011`: `tcp request content` in HTTPS with RSA key (<255 req/s).
* `8021`: `tcp request content` in HTTPS with ECDSA key (2.4k req/s).
* `8002`: `tcp request session` in HTTP (183k req/s).
* `8012`: `tcp request session` in HTTPS with RSA key (<255 req/s).
* `8022`: `tcp request session` in HTTPS with ECDSA key (2.5k req/s).
* `8003`: `tcp request connection` in HTTP (185k req/s).
* `8013`: `tcp request connection` in HTTPS with RSA key (81k req/s).
* `8023`: `tcp request connection` in HTTPS with ECDSA key (81k req/s).

The sooner the connection is closed (`tcp-request connection`) the
better the performances are. This is an expected result because
HAProxy does not have to process the request.

Same goes for HTTPS, if the connection is closed before the session is
established, there is no need to exchange keys.
