# Miscellaneous optimizations

This directory contains miscellaneous optimizations bulk examples.

## Multiline ACL

In HAProxy the same ACL can be defined several times. This type of
definition will consider each lines as logical OR:


In following example a source IP address matches the `allowed_host` if
is from `10.0.10.10` or `10.0.10.20` or `10.0.10.30` or listed in
`office-fr.acl` or in `office-hr.acl` and so on.

```
acl allowed_host src 10.0.10.10 10.0.10.20 10.0.10.30
acl allowed_host src -m ip -f office-fr.acl
acl allowed_host src -m ip -f office-hr.acl
acl allowed_host src -m ip -f office-us.acl
acl allowed_host src -m ip -f office-ca.acl
```

in this case 5 lookup trees will be created, one for each line having
a substential impact on performances. This ACL can be rewritten in one
single line to increase the performances:

```
acl allowed_host src -f office-fr.acl -f office-hr.acl -f office-us.acl -f office-ca.acl 10.0.10.10 10.0.10.20 10.0.10.30
```

Run injectors using following commands to see the impact:

```
taskset -c 1 ./h1load -t 1 -c 40 -d 20 -S 'http://198.18.0.102:8001/'
taskset -c 1 ./h1load -t 1 -c 40 -d 20 -S 'http://198.18.0.102:8002/'
```

* Multi lines ACL: 132k req/s
* Single line ACL: 134k req/s

## X-Forwarded-For and X-Forwarded-Proto

`x-forwarded-for` header can be used to pass the client IP to the
backend servers. This header has no validation and can be a security
flaw. Basically you shouldn't trust existing `x-forwarded-for` header
and remove all of them.

A `x-forwarded-for` header can be added by either `option forwardfor`
or `http-request set-header`. In addition to `option forwardfor`,
`http-request del-header x-forwarded-for` can be used to remove all
previous settings.

Performances of all combinaisons can be benchmarked using ports 8101
to 8105.

A rule of thumb is for best performances and security ratio use
`option forwardfor` in combinaison to `http-request del-header
x-forwarded-for` (frontend `xforwardedfor-option-del` port 8105).

In the same vain `x-forwarded-proto` can be used to inform the backend
server the request was made over HTTPS. The traditional way to do this
was to add 2 `http-request` rule (see `xforwardedproto` frontend port
8106). The `iif` converter can be used to save one `http-request` rule
and increase performances (see `xforwardedproto-iif` frontend port
8107).

## HTTP and HTTPS

Instead of defining 2 different frontend with the same configuration
both HTTP and HTTPS frontends can be merged into one:

```
frontend web
    bind *:80
    bind *:443 ssl crt /etc/haproxy/certs
    # full ruleset
    use_backend my-backend
```

Note: This also work is the HTTP frontend is a single redirection:

```
frontend http
    bind *:80
    redirect scheme https code 301
```

In term of redirection `redirect scheme` should be discarded in favor
of `http-request redirect scheme`. The reason is that `redirect` rule
is evaluated after all `http-request` ones. In other words if a
backend has a lot of `http-rules` the redirection from HTTP to HTTPS
will take time and decrease performances. An other reason is that
`http-request redirect` rules are more flexible that old-style
`redirect`.

HTTP to HTTPS redirection can be specified like:

```
http-request redirect scheme https code 301 if !{ ssl_fc }
```

However this can lead to unexpected results if `http-requests` rules
are used. The configuration parser display a warning about that:

```
[WARNING]  (24542) : config : parsing [haproxy.cfg:240] : a 'http-request' rule placed after a 'redirect' rule will still be processed before.
```

## last_rule_file and last_rule_line

These new fetch sample can be added into the log format to see where
is located the last decision rule used when the request has been
processed.

```
# Add last rule information to standard log line.
# HTTP_LOG should be declared in the global section using a
# setenv directive.
log-format "${HTTP_LOG} lr:%[last_rule_file]:%[last_rule_line]"
```

Example of an issued line:
```
Nov  2 17:08:27 lab-inj02 haproxy[24499]: 198.18.0.101:47082 [02/Nov/2022:17:08:27.161] xforwardedfor-option-del demo-server/<NOSRV> 0/-1/-1/-1/0 200 74 - - LR-- 1/1/0/0/0 0/0 "GET / HTTP/1.1" cpu_ns_avg:72us lr:haproxy.cfg:97
```

Note for this example to work make sure `HTTP_LOG` environment
variable is defined in the `global` section:

```
setenv HTTP_LOG "%ci:%cp [%tr] %ft %b/%s %TR/%Tw/%Tc/%Tr/%Ta %ST %B %CC %CS %tsc %ac/%fc/%bc/%sc/%rc %sq/%bq %hr %hs %{+Q}r"
```

## Conditional configuration

Some parts of a configuration file can be activated using `.if` and
`.endif` directives. These directives acts like `#ifdef` and `#endif`
C directives.

All configuration files used in this workshop use this feature to
activate some features such as loggin or the pattern cache.

## String quoting

Since HAProxy 1.6 strings can be quoted. This allows to have spaces
without having to escape them.

```
# Without string quoting
http-request set-header x-comment this\ is difficult\ to\ read
# With string quoting
http-request set-header x-comment "this is easier to read"
```

Please refer to *Quoting and escaping* section in official HAProxy
documentation.
